/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einfache Programmieraufgaben mit mehreren Teilaufgaben
*/
function LmsConnector(
    node        // DOM-Knoten, der für die Benutzeroberfläche genutzt werden soll
) {
    this.upgradeLicenceBtn();
    this._ui = new TaskUI(node);
    this.connect();
    this._ui.onlevelunlock = (() => this.saveScore());
    this._ui.onnewfine = (() => this.saveScore());
}

// Genutzte TaskUI-Instanz
LmsConnector.prototype._ui = null;

/**
* Schaltfläche zum Einblenden der Lizenz-Infos anzeigen
* @return undefined
*/
LmsConnector.prototype.upgradeLicenceBtn = function() {
    new ScoTools.InfoPageLink();
}

/**
* Zufallszahl generieren, die als festes Start-Seed für die Pseudozufallszahlengenerierung verwendet wird
* @return Number    Seed
*/
LmsConnector.genSeed = function() {
    return Math.floor(Math.random() * 233280);
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
LmsConnector.prototype.connect = function() {
    if (typeof ScoTools.LMSLink == 'function') {
        var self = this;
        this.lmslink = new ScoTools.LMSLink();
        this.lmslink.onnew = (function() {
            self.lmsValuesInit();
        });
        this.lmslink.start();
        // Werte laden
        try {
            const lessonLocation = this.lmslink.lessonLocation;
            const data = JSON.parse('[' + lessonLocation + ']');
            if (data.length < 1)
                data.push(0);
            if (data.length < 2)
                data.push(0);
            if (data.length < 3)
                data.push(LmsConnector.genSeed());
            this._ui.loadValues(this.getDone(), +data[0], +data[1], +data[2]);
            const newDataStr = data.join(',');
            if (newDataStr != lessonLocation)
                this.lmslink.lessonLocation = newDataStr;
        } catch(e) {
            const data = [0, 0, LmsConnector.genSeed()];
            if (this.canSave())
                this._ui.loadValues(false, data[0], data[1], data[2]);
            this.lmslink.lessonLocation = data.join(',');
        }
    }
    if (!this.lmslink.ready)
        this._ui.addGeneralMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn');
}

/**
* Beim ersten Start des Lernpaketes Anfangswerte festlegen
* @return undefined
*/
LmsConnector.prototype.lmsValuesInit = function() {
    this.lmslink.lessonLocation = '0,0,' + LmsConnector.genSeed();
}

// Genutzte Instanz von LMSLink
LmsConnector.prototype.lmslink = { 'ready': false, lessonLocation: '0,0,' + LmsConnector.genSeed() };

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
LmsConnector.prototype.getDone = function() {
    return (this.lmslink.ready && !this.lmslink.isIncomplete());
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
LmsConnector.prototype.canSave = function() {
    return (!this.lmslink.ready || this.lmslink.isNormalIncomplete());
}

/**
* Bearbeitungszeit speichern
* @return undefined
*/
LmsConnector.prototype.reportTime = function() {
    this.lmslink.ready && this.lmslink.reportTime();
}

/**
* Punktestand und Status speichern
* @return undefined
*/
LmsConnector.prototype.saveScore = function() {
    if (!this.canSave()) {
        this.reportTime();
        return;
    }
    const data = {
        lessonLocation: (Number.isFinite(this._ui.active) ? this._ui.active : 0) + ',' + this._ui.fineCount + ',' + this._ui.startSeed,
        lessonStatus: this._ui.done ? 'completed' : 'incomplete',
        scoreMin: 0,
        scoreMax: 100,
        score: this._ui.done ? Math.max(51, 100 - this._ui.fineCount) : Math.min(49, this._ui.fineCount)
    };
    this.save(data);
}

/**
* Einen Wert vom LMS lesen
* @return undefined
*/
LmsConnector.prototype.read = function(
    key     // Attributname von LMSLink-Instanz, die gelesen werden soll
) {
    return (this.lmslink.ready ? this.lmslink[key] : undefined);
}

/**
* Einen Wert vom LMS speichern lassen
* @return undefined
*/
LmsConnector.prototype.write = function(
    key,        // Attributname von LMSLink-Instanz, unter dem der Wert gespeichert werden soll
    value       // Zu speichernder Wert
) {
    if (this.lmslink.ready)
        this.lmslink[key] = value;
}

/**
* Zugleich mehrere Werte ändern, die Zeit speichern und dann erst Commit an das LMS senden
* @return undefined
*/
LmsConnector.prototype.save = function(
    data        // Objekt mit zu speichernden Daten
) {
    if (this.lmslink.ready)
        this.lmslink.save(data);
}
