/** Represents a scope within a TableOfVars instance */
function TableOfVarsInstance(
    parentTable,    // TableOfVarsAbstract instance containing this instance
    classInfo,      // Object like { className: 'Internal class name', isa: "Optional array of class names beeing anchestors or implemented by this class", atts: "Array of attributes suitable for addVars method" }
    pointer         // Optional: Preferred memory address
) {
    this.className = classInfo.className;
    this.calcSize(classInfo.atts);
    TableOfVarsInstance.parent.call(this, parentTable, pointer);
    if (classInfo.atts instanceof Array)
        this.addVars(classInfo.atts);
}
TableOfVarsInstance.parent = TableOfVarsInstancelike;
TableOfVarsInstance.prototype = Object.create(TableOfVarsInstance.parent.prototype);
TableOfVarsInstance.prototype.constructor = TableOfVarsInstance;

/** Internal class name */
Object.defineProperty(TableOfVarsInstance.prototype, 'className', {
    enumerable: true,
    get() { return this.baseType; },
    set(v) { this.baseType = v; }
});

/**
* Set this.size by calculating size of attributes
* @return undefined
*/
TableOfVarsInstance.prototype.calcSize = function(
    atts        // Array of attributes suitable for addVars method
) {
    this.size = atts.map(({ type, dimensions }) => {
        if (dimensions)
            return VarTypeReference.countBytes;
        const varClass = this.varTypes.reduce((prev, curr) => (prev || ((curr.typeName == type) ? curr : null)), null);
        return varClass ? varClass.countBytes : VarTypeReference.countBytes;
    }).reduce((a, b) => a + (Number.isFinite(b) ? Math.max(0, b) : VarType.countBytes), 0);
    if (this.size < 1)
        this.size = 1;
}

/**
* Start a new scope referencing this instance
* @return undefined
*/
TableOfVarsInstance.prototype.startScope = function() {
    this.addScope(this.pointer);
}

/** Description of this type in error messages */
TableOfVarsInstance.prototype.descType = 'Objekt';

/**
* Convert contained data to JSON
* @return Object    JSON representation of contained data
*/
TableOfVarsInstance.prototype.toJSON = function() {
    return {
        instance: 'object',
        className: this.baseType,
        atts: this.vars.map(variable => variable.toJSON()),
        pointer: this.pointer
    };
}
